use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::BabelCall;
use crate::types::GetAffiliatedKeywords;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmBabelCall {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) call: Option<String>,
    #[serde(rename = "inside-header")]
    pub(crate) inside_header: Option<String>,
    pub(crate) arguments: Option<String>,
    #[serde(rename = "end-header")]
    pub(crate) end_header: Option<String>,
    pub(crate) value: String,
}

to_wasm!(
    WasmBabelCall,
    BabelCall<'s>,
    original,
    wasm_context,
    { WasmAstNode::BabelCall(original) },
    { "babel-call".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        Ok((
            Vec::new(),
            WasmBabelCall {
                additional_properties,
                call: original.call.map(|s| s.to_owned()),
                inside_header: original.inside_header.map(|s| s.to_owned()),
                arguments: original.arguments.map(|s| s.to_owned()),
                end_header: original.end_header.map(|s| s.to_owned()),
                value: original.value.to_owned(),
            },
        ))
    }
);
