use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::CommentBlock;
use crate::types::GetAffiliatedKeywords;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmCommentBlock {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) value: String,
}

to_wasm!(
    WasmCommentBlock,
    CommentBlock<'s>,
    original,
    wasm_context,
    { WasmAstNode::CommentBlock(original) },
    { "comment-block".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        Ok((
            Vec::new(),
            WasmCommentBlock {
                additional_properties,
                value: original.contents.to_owned(),
            },
        ))
    }
);
