use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::headline::Noop;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::FootnoteDefinition;
use crate::types::GetAffiliatedKeywords;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmFootnoteDefinition {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) label: String,
    #[serde(rename = "pre-blank")]
    pub(crate) pre_blank: Noop,
}

to_wasm!(
    WasmFootnoteDefinition,
    FootnoteDefinition<'s>,
    original,
    wasm_context,
    { WasmAstNode::FootnoteDefinition(original) },
    { "footnote-definition".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((
            children,
            WasmFootnoteDefinition {
                additional_properties,
                label: original.label.to_owned(),
                pre_blank: Noop {},
            },
        ))
    }
);
