use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::GetAffiliatedKeywords;
use crate::types::HorizontalRule;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmHorizontalRule {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
}

to_wasm!(
    WasmHorizontalRule,
    HorizontalRule<'s>,
    original,
    wasm_context,
    { WasmAstNode::HorizontalRule(original) },
    { "horizontal-rule".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        Ok((
            Vec::new(),
            WasmHorizontalRule {
                additional_properties,
            },
        ))
    }
);
