use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use crate::types::LinkType;
use crate::types::PlainLink;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "format")]
#[serde(rename = "plain")]
pub struct WasmPlainLink {
    #[serde(rename = "type")]
    pub(crate) link_type: String,
    pub(crate) path: String,
    #[serde(rename = "raw-link")]
    pub(crate) raw_link: String,
    pub(crate) application: Option<String>,
    #[serde(rename = "search-option")]
    pub(crate) search_option: Option<String>,
}

to_wasm!(
    WasmPlainLink,
    PlainLink<'s>,
    original,
    wasm_context,
    { WasmAstNode::PlainLink(original) },
    { "link".into() },
    {
        Ok((
            Vec::new(),
            WasmPlainLink {
                link_type: match &original.link_type {
                    LinkType::File => "file".to_owned(),
                    LinkType::Protocol(protocol) => protocol.clone().into_owned(),
                    LinkType::Id => "id".to_owned(),
                    LinkType::CustomId => "custom-id".to_owned(),
                    LinkType::CodeRef => "coderef".to_owned(),
                    LinkType::Fuzzy => "fuzzy".to_owned(),
                },
                path: original.path.to_owned(),
                raw_link: original.raw_link.to_owned(),
                application: original.application.map(str::to_owned),
                search_option: original.search_option.map(str::to_owned),
            },
        ))
    }
);
