use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::GetAffiliatedKeywords;
use crate::types::QuoteBlock;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmQuoteBlock {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
}

to_wasm!(
    WasmQuoteBlock,
    QuoteBlock<'s>,
    original,
    wasm_context,
    { WasmAstNode::QuoteBlock(original) },
    { "quote-block".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((
            children,
            WasmQuoteBlock {
                additional_properties,
            },
        ))
    }
);
