use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::headline::Noop;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::types::GetAffiliatedKeywords;
use crate::types::SpecialBlock;
use crate::util::elisp_fact::ElispFact;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmSpecialBlock {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    #[serde(rename = "type")]
    pub(crate) block_type: String,
    pub(crate) parameters: Option<String>,
    pub(crate) results: Noop,
}

to_wasm!(
    WasmSpecialBlock,
    SpecialBlock<'s>,
    original,
    wasm_context,
    { WasmAstNode::SpecialBlock(original) },
    { "special-block".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((
            children,
            WasmSpecialBlock {
                additional_properties,
                block_type: original.block_type.to_owned(),
                parameters: original.parameters.map(|s| s.to_owned()),
                results: Noop {},
            },
        ))
    }
);
